package de.mlessmann.certassist.keystore;

import de.mlessmann.certassist.except.JavaSecurityException;
import java.io.IOException;
import java.nio.file.Path;
import java.security.KeyStore;
import java.security.KeyStoreException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;

public interface KeystoreUsage extends AutoCloseable {
    Path truststorePath();

    @Override
    default void close() {
        // Default implementation does nothing - overwrite this if you need to close resources.
    }

    default KeyStore readAsKeystore(String passphrase) throws JavaSecurityException {
        try {
            return KeyStore.getInstance(truststorePath().toFile(), passphrase.toCharArray());
        } catch (KeyStoreException | IOException | NoSuchAlgorithmException | CertificateException e) {
            throw new JavaSecurityException(e);
        }
    }
}
