package de.mlessmann.certassist.openssl;

import de.mlessmann.certassist.service.PassphraseService;
import java.util.Map;
import java.util.Objects;
import java.util.Optional;
import java.util.concurrent.ConcurrentHashMap;
import lombok.extern.slf4j.Slf4j;

@Slf4j
public class InMemoryCertificatePasswordProvider implements CertificatePasswordProvider {

    private final Map<String, String> passwords = new ConcurrentHashMap<>();
    private static final int PASSPHRASE_LENGTH = 16;

    @Override
    public String generateNewPassword() {
        return PassphraseService.generateNewPassword(PASSPHRASE_LENGTH);
    }

    @Override
    public String getPasswordFor(String certificateFingerprint) {
        log.trace("Getting password for certificate fingerprint: {}", certificateFingerprint);
        return Optional.ofNullable(passwords.get(certificateFingerprint)).orElseThrow();
    }

    @Override
    public void setPasswordFor(String certificateFingerprint, String password) {
        log.trace("Setting password for certificate fingerprint: {}", certificateFingerprint);
        Objects.requireNonNull(certificateFingerprint);
        Objects.requireNonNull(password);
        passwords.put(certificateFingerprint, password);
    }
}
